@@ -70,9 +70,13 @@ int usb_claim_interface(usb_dev_handle *dev, int interface)
   int ret;
 
   ret = ioctl(dev->fd, IOCTL_USB_CLAIMINTF, &interface);
-  if (ret < 0)
+  if (ret < 0) {
+    if (errno == EBUSY && usb_debug > 0)
+      fprintf(stderr, "Check that you have permissions to write to %s/%s and, if you don't, that you set up hotplug (http://linux-hotplug.sourceforge.net/) correctly.\n", dev->bus->dirname, dev->device->filename);
+
     USB_ERROR_STR(-errno, "could not claim interface %d: %s", interface,
 	strerror(errno));
+  }
 
   dev->interface = interface;
 
@@ -195,8 +199,9 @@ int usb_control_msg(usb_dev_handle *dev, int requesttype, int request,
   return ret;
 }
 
-int usb_find_devices_on_bus(struct usb_bus *bus)
+int usb_find_devices_on_bus(struct usb_bus *bus, struct usb_device **devices)
 {
+  struct usb_device *fdev = NULL;
   DIR *dir;
   struct dirent *entry;
   char dirpath[PATH_MAX + 1];
@@ -211,23 +216,12 @@ int usb_find_devices_on_bus(struct usb_bus *bus)
   while ((entry = readdir(dir)) != NULL) {
     char filename[PATH_MAX + 1];
     struct usb_device *dev;
-    int fd, ret, skip = 0;
+    int i, fd, ret;
 
     /* Skip anything starting with a . */
     if (entry->d_name[0] == '.')
       continue;
 
-    /* Don't readd it if we were called multiple times */
-    for (dev = bus->devices; dev; dev = dev->next) {
-      if (!strcmp(dev->filename, entry->d_name)) {
-        skip = 1;
-        break;
-      }
-    }
-
-    if (skip)
-      continue;
-
     dev = malloc(sizeof(*dev));
     if (!dev)
       USB_ERROR(-ENOMEM);
@@ -251,30 +245,99 @@ int usb_find_devices_on_bus(struct usb_bus *bus)
     }
 
     ret = read(fd, (void *)&dev->descriptor, sizeof(dev->descriptor));
-
-    close(fd);
-
     if (ret < 0) {
       if (usb_debug)
         fprintf(stderr, "usb_find_devices_on_bus: Couldn't read descriptor\n");
 
       free(dev);
-      continue;
+
+      goto err;
     }
 
-    if (bus->devices) {
-      dev->next = bus->devices;
-      dev->next->prev = dev;
-    }
-    bus->devices = dev;
+    LIST_ADD(fdev, dev);
 
     if (usb_debug >= 2)
       fprintf(stderr, "usb_find_devices_on_bus: Found %s on %s\n",
 		dev->filename, bus->dirname);
+
+    /* Now try to fetch the rest of the descriptors */
+    if (dev->descriptor.bNumConfigurations > USB_MAXCONFIG)
+      /* Silent since we'll try again later */
+      goto err;
+
+    if (dev->descriptor.bNumConfigurations < 1)
+      /* Silent since we'll try again later */
+      goto err;
+
+    dev->config = (struct usb_config_descriptor *)malloc(dev->descriptor.bNumConfigurations * sizeof(struct usb_config_descriptor));
+    if (!dev->config)
+      /* Silent since we'll try again later */
+      goto err;
+
+    memset(dev->config, 0, dev->descriptor.bNumConfigurations *
+          sizeof(struct usb_config_descriptor));
+
+    for (i = 0; i < dev->descriptor.bNumConfigurations; i++) {
+      char buffer[8], *bigbuffer;
+      struct usb_config_descriptor *desc = (struct usb_config_descriptor *)buffer;
+
+      /* Get the first 8 bytes so we can figure out what the total length is */
+      ret = read(fd, (void *)buffer, 8);
+      if (ret < 8) {
+        if (usb_debug >= 1) {
+          if (ret < 0)
+            fprintf(stderr, "Unable to get descriptor (%d)\n", ret);
+          else
+            fprintf(stderr, "Config descriptor too short (expected %d, got %d)\n", 8, ret);
+        }
+
+        goto err;
+      }
+
+      USB_LE16_TO_CPU(desc->wTotalLength);
+
+      bigbuffer = malloc(desc->wTotalLength);
+      if (!bigbuffer) {
+        if (usb_debug >= 1)
+          fprintf(stderr, "Unable to allocate memory for descriptors\n");
+        goto err;
+      }
+
+      /* Copy over the first 8 bytes we read */
+      memcpy(bigbuffer, buffer, 8);
+
+      ret = read(fd, (void *)(bigbuffer + 8), desc->wTotalLength - 8);
+      if (ret < desc->wTotalLength - 8) {
+        if (usb_debug >= 1) {
+          if (ret < 0)
+            fprintf(stderr, "Unable to get descriptor (%d)\n", ret);
+          else
+            fprintf(stderr, "Config descriptor too short (expected %d, got %d)\n", desc->wTotalLength, ret);
+        }
+
+        free(bigbuffer);
+        goto err;
+      }
+
+      ret = usb_parse_configuration(&dev->config[i], bigbuffer);
+      if (usb_debug >= 2) {
+        if (ret > 0)
+          fprintf(stderr, "Descriptor data still left\n");
+        else if (ret < 0)
+          fprintf(stderr, "Unable to parse descriptors\n");
+      }
+
+      free(bigbuffer);
+    }
+
+err:
+    close(fd);
   }
 
   closedir(dir);
 
+  *devices = fdev;
+
   return 0;
 }
 
